The tte_source object is used to define events and possible censorings.
Usage
tte_source(
dataset_name,
filter = NULL,
date,
censor = 0,
set_values_to = NULL,
order = order,
consider_end_dates = TRUE
)Arguments
- dataset_name
-
The name of the source dataset
The name refers to the dataset provided by the
source_datasetsparameter ofderive_param_tte().- Default value
none
- filter
-
An unquoted condition for selecting the observations from
datasetwhich are events or possible censoring time points.- Default value
NULL
- date
-
A variable or expression providing the date of the event or censoring. A date, or a datetime can be specified. An unquoted symbol or expression is expected.
Refer to
derive_vars_dt()orconvert_dtc_to_dt()to impute and derive a date from a date character vector to a date object.- Default value
none
- censor
-
Censoring value
CDISC strongly recommends using
0for events and positive integers for censoring.- Default value
0
- set_values_to
-
A named list returned by
exprs()defining the variables to be set for the event or censoring, e.g.exprs(EVENTDESC = "DEATH", SRCDOM = "ADSL", SRCVAR = "DTHDT"). The values must be a symbol, a character string, a numeric value, an expression, orNA.- Default value
NULL
- order
-
Sort order
An optional named list returned by
exprs()defining additional variables that the source dataset is sorted on afterdate.- Permitted values
list of variables created by
exprs()e.g.exprs(ASEQ).- Default value
order
- consider_end_dates
-
Should end dates be considered?
If end dates are considered, the records which are after the end date are ignored and the censor value specified for the end date takes precedence.
- Permitted values
TRUE,FALSE- Default value
TRUE